"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateBabelConfig = void 0;
const plugin_1 = __importDefault(require("vue-loader/dist/plugin"));
const compression_webpack_plugin_1 = __importDefault(require("compression-webpack-plugin"));
const path_1 = require("path");
const terser_webpack_plugin_1 = __importDefault(require("terser-webpack-plugin"));
const client_plugin_1 = __importDefault(require("vue-server-renderer/client-plugin"));
const server_plugin_1 = __importDefault(require("vue-server-renderer/server-plugin"));
const webpack_1 = require("webpack");
const webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
const packageJsonGenerator_1 = __importDefault(require("./customPlugins/packageJsonGenerator"));
const PackerPlugin_1 = __importDefault(require("./customPlugins/PackerPlugin"));
const ErrorFormatter_1 = __importDefault(require("./ErrorFormatter"));
const NONE = Symbol('none');
function clearObj(obj) {
    let resObj = {};
    for (let key in obj) {
        let value = obj[key];
        if (value !== NONE)
            resObj[key] = value;
    }
    return resObj;
}
function cleanIdentifier(ident) {
    return ident.substr(ident.lastIndexOf('!') + 1).replace(/^multi /, '');
}
function chunkById(chunks, chunkId) {
    for (let chunk of chunks)
        if (chunk.id === chunkId)
            return chunk;
    return { files: [] };
}
exports.generateBabelConfig = (shitSupport) => {
    let data = {
        "presets": [[require.resolve('@babel/preset-env'), shitSupport ? {
                    "targets": ">100%",
                    modules: false,
                    useBuiltIns: 'usage',
                    corejs: 3,
                } : {
                    "targets": {
                        "browsers": ">50%",
                        "node": "current"
                    },
                    shippedProposals: true,
                    modules: false
                }]],
        "plugins": []
    };
    data.plugins.push(require.resolve('@babel/plugin-syntax-dynamic-import'));
    data.plugins.push([require.resolve('@babel/plugin-transform-typescript'), {
            isTSX: true
        }]);
    data.plugins.push(require.resolve('@babel/plugin-proposal-object-rest-spread'));
    data.plugins.push(require.resolve('@babel/plugin-proposal-do-expressions'));
    data.plugins.push([require.resolve('@babel/plugin-proposal-decorators'), {
            legacy: true
        }]);
    data.plugins.push(require.resolve('@babel/plugin-proposal-class-properties'));
    data.plugins.push(require.resolve('@babel/plugin-proposal-nullish-coalescing-operator'));
    data.plugins.push(require.resolve('@babel/plugin-proposal-optional-chaining'));
    return data;
};
async function mkConfig(config) {
    let aliases = {
        ...config.moduleAliases
    };
    let singleChunk = !!process.env.SINGLE_CHUNK;
    let shitSupport = !!process.env.SHIT_SUPPORT;
    let forVue = false;
    if (config.for.startsWith('vue-')) {
        forVue = true;
        config.for = config.for.slice(4);
    }
    if (config.for === 'node' && forVue)
        singleChunk = true;
    if (config.for === 'web') {
        aliases = {
            ...aliases,
            'crypto': 'crypto-browserify',
            'stream': 'stream-browserify',
            'vm': 'vm-browserify',
            'path': 'path-browserify',
            'http': 'stream-http',
            'process': 'node-process',
            'fs': require.resolve('./stubs/fs'),
            'child_process': require.resolve('./stubs/child_process'),
        };
    }
    let ret = {
        // Web target is selected for extensions
        target: config.for,
        node: {
            __dirname: false,
            __filename: false,
        },
        module: {
            rules: [],
            unknownContextCritical: false
        },
        experiments: {
            mjs: true,
            asyncWebAssembly: true,
            importAwait: true,
            importAsync: true,
            topLevelAwait: true,
        },
        devtool: config.env === 'development' ? 'source-map' : 'hidden-source-map',
        entry: {},
        resolve: {
            alias: aliases,
            extensions: [
                '.tsx', '.ts', '.mjs', '.jsx', '.js', '.json', '.yml', '.xml', '.vue'
            ],
            modules: [...[
                    path_1.join(config.projectBaseDir, 'node_modules'),
                    (config.for === 'web' || config.for === 'webworker') ? path_1.join(__dirname, '../browserLibs/node_modules/') : null,
                    path_1.join(__dirname, '../commonLibs/node_modules/'),
                ].filter(m => m !== null).map(path_1.normalize), 'node_modules'],
            mainFields: [],
            symlinks: true,
        },
        output: clearObj({
            sourcePrefix: '',
            filename: singleChunk ? 'main.js' : (config.env === 'development' ? NONE : '[chunkhash].js'),
            sourceMapFilename: config.env === 'development' ? NONE : '[file].map[query]',
            path: config.outputDirPath,
            chunkFilename: config.env === 'development' ? NONE : 'c.[chunkhash].js',
            jsonpFunction: config.env === 'development' ? NONE : 'zarbisOnLoad',
            pathinfo: config.env === 'development',
            devtoolModuleFilenameTemplate: '[absolute-resource-path]',
            devtoolFallbackModuleFilenameTemplate: `[absolute-resource-path]?[loaders]`,
            libraryTarget: (config.for === 'node' ? 'commonjs2' : NONE),
            publicPath: config.for === 'web' ? '/' : NONE,
            crossOriginLoading: 'anonymous'
        }),
        plugins: [],
        externals: [
            'zarbis'
        ],
        stats: {
            errorDetails: true
        },
        cache: true,
        parallelism: 7,
        mode: 'none',
        optimization: {
            chunkIds: config.env === 'production' ? 'natural' : 'named',
            moduleIds: config.env === 'production' ? 'natural' : 'named',
            mangleExports: config.env === 'production',
            removeAvailableModules: config.env === 'production',
            removeEmptyChunks: config.env === 'production',
            mergeDuplicateChunks: config.env === 'production',
            flagIncludedChunks: config.env === 'production',
            sideEffects: config.env === 'production',
            providedExports: config.env === 'production',
            usedExports: config.env === 'production',
            concatenateModules: config.env === 'production',
            noEmitOnErrors: false,
            portableRecords: true,
            minimize: !process.env.NO_TERSER && config.env === 'production',
            minimizer: process.env.NO_TERSER ? [] : [
                new terser_webpack_plugin_1.default({
                    sourceMap: true,
                    extractComments: {
                        condition: false,
                    },
                    terserOptions: {
                        parse: {
                            bare_returns: false,
                            ecma: 8,
                            expression: false,
                            filename: null,
                            html5_comments: true,
                            shebang: true,
                            strict: false,
                            toplevel: null
                        },
                        compress: {
                            arrows: true,
                            booleans: true,
                            collapse_vars: true,
                            comparisons: true,
                            computed_props: true,
                            conditionals: true,
                            dead_code: true,
                            drop_console: false,
                            drop_debugger: true,
                            ecma: 8,
                            evaluate: true,
                            expression: false,
                            global_defs: {},
                            hoist_funs: false,
                            hoist_props: true,
                            hoist_vars: false,
                            ie8: false,
                            if_return: true,
                            inline: true,
                            join_vars: true,
                            keep_classnames: false,
                            keep_fargs: true,
                            keep_fnames: false,
                            keep_infinity: false,
                            loops: true,
                            negate_iife: true,
                            passes: 1,
                            properties: true,
                            pure_getters: "strict",
                            pure_funcs: null,
                            reduce_funcs: true,
                            reduce_vars: true,
                            sequences: true,
                            side_effects: true,
                            switches: true,
                            top_retain: null,
                            toplevel: false,
                            typeofs: true,
                            unsafe: false,
                            unsafe_arrows: false,
                            unsafe_comps: false,
                            unsafe_Function: false,
                            unsafe_math: false,
                            unsafe_methods: false,
                            unsafe_proto: false,
                            unsafe_regexp: false,
                            unsafe_undefined: false,
                            unused: true,
                            warnings: false
                        },
                        mangle: {
                            eval: false,
                            ie8: false,
                            keep_classnames: false,
                            keep_fnames: false,
                            properties: false,
                            safari10: false,
                            toplevel: false
                        },
                        output: {
                            ascii_only: false,
                            beautify: false,
                            comments: false,
                            ecma: 8,
                            ie8: false,
                            indent_level: 4,
                            indent_start: 0,
                            inline_script: true,
                            keep_quoted_props: false,
                            max_line_len: false,
                            preamble: null,
                            quote_keys: false,
                            quote_style: 0,
                            safari10: false,
                            semicolons: true,
                            shebang: true,
                            source_map: null,
                            webkit: false,
                            width: 80,
                            wrap_iife: false
                        },
                        wrap: false
                    }
                })
            ],
            splitChunks: {
                chunks: "all", maxInitialRequests: 30, maxAsyncRequests: 30, maxSize: 100000
            },
            runtimeChunk: (forVue && config.for === 'web') ? 'single' : false,
        }
    };
    // Main fields
    {
        let mainFields = ['module', 'main'];
        if (config.env === 'development')
            mainFields = ['dev:module', ...mainFields];
        if (config.for !== 'node') {
            mainFields = ['browser', ...mainFields];
            if (config.env === 'development')
                mainFields = ['dev:browser', ...mainFields];
        }
        mainFields = [...mainFields];
        ret.resolve.mainFields = mainFields;
    }
    // Module rules
    {
        ret.module.rules.push({
            test: /node_modules[\/\\](iconv-lite)[\/\\].+/,
            resolve: {
                aliasFields: ['main']
            }
        });
        ret.module.rules.push({
            type: 'javascript/auto',
            test: /\.mjs$/,
            use: []
        });
        ret.module.rules.push({
            test: /\.(c|le)ss$/,
            loader: require.resolve('vue-style-loader'),
        });
        ret.module.rules.push({
            test: /\.(c|le)ss$/,
            loader: require.resolve('css-loader'),
            options: {
                modules: {
                    mode: 'local',
                    exportGlobals: true,
                    localIdentName: config.env === 'development' ? '[path][name]__[local]--[hash:base64:5]' : '[hash:base64:12]',
                },
                sourceMap: config.env === 'development',
                localsConvention: 'camelCaseOnly'
            }
        });
        if (config.env === 'production') {
            ret.module.rules.push({
                test: /\.(c|le)ss$/,
                loader: require.resolve('postcss-loader'),
                options: {
                    plugins(loader) {
                        return [
                            require('postcss-preset-env')(),
                            require('cssnano')(),
                        ];
                    }
                },
            });
        }
        ret.module.rules.push({
            test: /\.less$/,
            loader: require.resolve("less-loader")
        });
        // Files
        ret.module.rules.push({
            test: /\.(txt|png|jpg|otf|gif|jpeg|svg|ttf|woff|eot|woff2)$/,
            loader: require.resolve('url-loader'),
            options: {
                esModule: false,
                fallback: require.resolve('file-loader'),
                limit: singleChunk ? Infinity : 8192,
                name: config.env === 'development' ? '[name].[sha512:hash:hex:20].[ext]' : '[sha512:hash:hex:20].[ext]'
            }
        });
        // Just js/jsx
        ret.module.rules.push({
            test: /\.(:?m?[jt]sx?)$/i,
            loader: require.resolve('babel-loader'),
            exclude: /\.min\.js$/,
            options: exports.generateBabelConfig(shitSupport),
        });
        ret.module.rules.push({
            test: /\.vue$/i,
            loader: require.resolve('vue-loader'),
        });
    }
    // Entry points
    for (let key of Object.keys(config.entryPoints)) {
        if (config.env === 'development')
            ret.entry[key] = [path_1.join(__dirname, 'customLoaders/hmrApplier.js'), ...config.entryPoints[key]];
        else
            ret.entry[key] = [...config.entryPoints[key]];
        if (config.for === 'node')
            ret.entry[key] = [path_1.join(__dirname, 'customLoaders/nodeSupport.js'), ...ret.entry[key]];
    }
    // Plugins
    {
        if (forVue) {
            if (config.for === 'node') {
                ret.plugins.push(new server_plugin_1.default());
            }
            else if (config.for === 'web') {
                ret.plugins.push(new client_plugin_1.default());
            }
            else {
                throw new Error('Unknown vue target: ' + config.for);
            }
        }
        ret.plugins.push(new plugin_1.default());
        ret.plugins.push(new ErrorFormatter_1.default(false, false));
        if (process.env.PACKER)
            ret.plugins.push(new PackerPlugin_1.default());
        ret.plugins.push(new webpack_1.DefinePlugin({
            'process.browser': config.for === 'web',
            'process.env.ELECTRON': config.for.startsWith('electron'),
            'process.env.BROWSER': config.for === 'web' || config.for === 'electron-renderer',
            'process.env.NODE': config.for === 'node' || config.for.startsWith('electron'),
            'process.env.NODE_ENV': JSON.stringify(config.env),
            'process.env.ENV': JSON.stringify(config.env),
            'process.env.ZARBIS_ALIASES': JSON.stringify(aliases)
        }));
        (config.for === 'node' || config.for.startsWith('electron')) && ret.plugins.push(new packageJsonGenerator_1.default({
            name: config.outTargetName
        }, config.projectBaseDir));
        (config.for === 'web' || config.for === 'webworker') && ret.plugins.push(new webpack_1.NormalModuleReplacementPlugin(/^http2$/, __dirname + '/stubs/http2.js'), new webpack_1.NormalModuleReplacementPlugin(/^cluster$/, __dirname + '/stubs/cluster.js'), new webpack_1.NormalModuleReplacementPlugin(/^fs$/, __dirname + '/stubs/fs.js'));
        for (let from of Object.keys(aliases)) {
            const regexp = new RegExp('^' + from + '(:?$|\\/)');
            ret.plugins.push(new webpack_1.NormalModuleReplacementPlugin(regexp, function (resource) {
                resource.request = resource.request.replace(regexp, aliases[from]);
            }));
        }
        // if (!forVue) {
        config.env === 'development' && ret.plugins.push(new webpack_1.HotModuleReplacementPlugin({
            multiStep: false,
        }));
        // }
        if (config.for === 'web') {
            ret.plugins.push(new webpack_1.ProvidePlugin({
                'process': 'process',
                'Buffer': ['buffer', 'Buffer']
            }));
        }
        if (config.env === 'production' && config.for === 'web')
            ret.plugins.push(new compression_webpack_plugin_1.default({
                minRatio: 1,
                threshold: 0,
                filename: '[path].gz',
                compressionOptions: {
                    numiterations: 15
                },
            }));
        config.env === 'production' && ret.plugins.push(new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
            analyzerMode: 'static',
            defaultSizes: 'gzip',
            openAnalyzer: false
        }));
        if (singleChunk) {
            ret.plugins.push(new webpack_1.optimize.LimitChunkCountPlugin({
                maxChunks: 1,
            }));
        }
    }
    // Externals
    const externals = [/^(:?@[a-z\-0-9]+\/)?[a-z\-0-9]+\/?/, ...config.externals];
    const internals = config.internals;
    config.for === 'node' && ret.externals.push('http2', ({ context, request }, callback) => {
        if (process.env.TRACE_EXTERNALS)
            console.log(`[EXT] ${request}`);
        if (internals.some(r => r.test(request))) {
            if (process.env.TRACE_EXTERNALS)
                console.log(`[EXT] => internal`);
            return callback();
        }
        else if (externals.some(r => r.test(request))) {
            if (process.env.TRACE_EXTERNALS)
                console.log(`[EXT] => external`);
            return callback(null, `commonjs ${request}`);
        }
        else {
            if (process.env.TRACE_EXTERNALS)
                console.log(`[EXT] => default (internal)`);
            return callback();
        }
    });
    return ret;
}
exports.default = mkConfig;
//# sourceMappingURL=webpackConfigGenerator.js.map